=== modified file 'lib/lp/services/job/interfaces/job.py'
--- lib/lp/services/job/interfaces/job.py	2011-05-19 15:15:16 +0000
+++ lib/lp/services/job/interfaces/job.py	2011-05-26 16:29:59 +0000
@@ -14,6 +14,7 @@
     'ITwistedJobSource',
     'JobStatus',
     'LeaseHeld',
+    'SuspendJobException',
     ]
 
 
@@ -35,6 +36,11 @@
 from canonical.launchpad import _
 
 
+class SuspendJobException(Exception):
+    """Raised when a running job wants to suspend itself."""
+    pass
+
+
 class LeaseHeld(Exception):
     """Raised when attempting to acquire a list that is already held."""
 

=== modified file 'lib/lp/services/job/model/job.py'
--- lib/lp/services/job/model/job.py	2011-05-19 15:15:16 +0000
+++ lib/lp/services/job/model/job.py	2011-05-26 14:29:34 +0000
@@ -78,6 +78,7 @@
         JobStatus.RUNNING:
             (JobStatus.COMPLETED,
              JobStatus.FAILED,
+             JobStatus.SUSPENDED,
              JobStatus.WAITING),
         JobStatus.FAILED: (),
         JobStatus.COMPLETED: (),

=== modified file 'lib/lp/services/job/runner.py'
--- lib/lp/services/job/runner.py	2011-05-19 15:15:16 +0000
+++ lib/lp/services/job/runner.py	2011-05-26 16:29:59 +0000
@@ -51,6 +51,7 @@
     IJob,
     IRunnableJob,
     LeaseHeld,
+    SuspendJobException,
     )
 from lp.services.mail.sendmail import MailController
 from lp.services.scripts.base import LaunchpadCronScript
@@ -182,6 +183,10 @@
                 self.logger.exception(
                     "Scheduling retry due to %s.", e.__class__.__name__)
                 do_retry = True
+        except SuspendJobException:
+            self.logger.debug("Job suspended itself")
+            job.suspend()
+            self.incomplete_jobs.append(job)
         except Exception:
             self.logger.exception("Job execution raised an exception.")
             transaction.abort()

=== modified file 'lib/lp/services/job/tests/test_job.py'
--- lib/lp/services/job/tests/test_job.py	2011-03-23 15:30:02 +0000
+++ lib/lp/services/job/tests/test_job.py	2011-05-27 08:18:07 +0000
@@ -169,9 +169,10 @@
             JobStatus.SUSPENDED)
 
     def test_suspend_when_running(self):
-        """When a job is running, attempting to suspend is invalid."""
+        """When a job is running, attempting to suspend is valid."""
         job = Job(_status=JobStatus.RUNNING)
-        self.assertRaises(InvalidTransition, job.suspend)
+        job.suspend()
+        self.assertEqual(JobStatus.SUSPENDED, job.status)
 
     def test_suspend_when_completed(self):
         """When a job is completed, attempting to suspend is invalid."""

=== modified file 'lib/lp/services/job/tests/test_runner.py'
--- lib/lp/services/job/tests/test_runner.py	2011-05-25 14:07:19 +0000
+++ lib/lp/services/job/tests/test_runner.py	2011-05-26 16:29:59 +0000
@@ -22,6 +22,7 @@
 from lp.services.job.interfaces.job import (
     IRunnableJob,
     JobStatus,
+    SuspendJobException,
     )
 from lp.services.job.model.job import Job
 from lp.services.job.runner import (
@@ -38,6 +39,7 @@
     TestCaseWithFactory,
     ZopeTestInSubProcess,
     )
+from lp.testing.fakemethod import FakeMethod
 from lp.testing.mail_helpers import pop_notifications
 
 
@@ -363,6 +365,17 @@
         runner.runJobHandleError(job)
         self.assertEqual(1, len(self.oopses))
 
+    def test_runJob_with_SuspendJobException(self):
+        # A job that raises SuspendJobError should end up suspended.
+        job = NullJob('suspended')
+        job.run = FakeMethod(failure=SuspendJobException())
+        runner = JobRunner([job])
+        runner.runJob(job)
+
+        self.assertEqual(JobStatus.SUSPENDED, job.status)
+        self.assertNotIn(job, runner.completed_jobs)
+        self.assertIn(job, runner.incomplete_jobs)
+
 
 class StuckJob(BaseRunnableJob):
     """Simulation of a job that stalls."""

